@@ -12,7 +12,7 @@
 ## x = environmental  scores (along one or two dimension)
 ## ext = extent along the environmental score. Vector including corresponding to c(xmin, xmax, ymin, ymax). The y component is optional.
 ## th = percentile threshold used to delineate the niche. 0 means that all occurences are included within the niche. 0.05 means that 95 % of the distribution is included within the niche.
-## env.mask = raster of vector containing the environmental background densities
+## env.mask = SpatRaster of vector containing the environmental background densities
 ## method = method used to draw the kernel density distribution. By default, 'adehabitat' uses kernelUD from the library adehabitatHR but you can set it to 'ks' to use the the algorithm from the ks library
 ## extend.extent = vector with extention values of the window size (see details)
 ##
@@ -23,7 +23,7 @@
 ## sp= scores for the occurrences of the species in the ordination, glob = scores for the whole studies areas, glob 1 = scores for the range of sp
 ## R= resolution of the grid, th.sp=quantile of species densitie at species occurences used as a threshold to exclude low species density values,
 ## th.env=The quantile used to delimit a threshold to exclude low environmental density values of the study area.
-## geomask= a geographical mask to delimit the background extent if the analysis takes place in the geographical space. It can be a SpatialPolygon or a raster object. Note that the CRS should be the same as the one used for the points.
+## geomask= a geographical mask to delimit the background extent if the analysis takes place in the geographical space. It can be a SpatialPolygon or a SpatRaster object. Note that the CRS should be the same as the one used for the points.
 ## kernel.method = method used to estimate the the kernel density. The initial and original method is 'adehabitat', while 'ks' has been recently implemented for future developments in multidimensional space
 ##
 ## dynamic.index(z1,z2,intersection=NA)
@@ -65,8 +65,8 @@
 ## bootstrap.rep : number of resamplings if the boostrap is selected. 100 is a minimum, 1000 would be better
 ## bootstrap.ncore = number of cores to use for parallelization if the boostrap estimation is selected
 ## it returns 3 objects
-## niche.density : a raster of the niche density
-## niche.envelope : a raster of the niche envelope in the environmental space. If boostrap is selected, it shows the percentages of confidence of the margin.
+## niche.density : a SpatRaster of the niche density
+## niche.envelope : a SpatRaster of the niche envelope in the environmental space. If boostrap is selected, it shows the percentages of confidence of the margin.
 ## niche.contour : a SpatialLine object delineating the contour of the niche. If bootstrap is selected, it represents the contour if the 95% confidence interval
 
 
@@ -87,12 +87,17 @@ ecospat.kd <- function(x, ext, R = 100, th = 0, env.mask = c(),
                                        h = "href", grid = mask,
                                        kern = "bivnorm"
       ) # calculate the density of occurrences in a grid of RxR pixels along the score gradients
-      x.dens <- raster::raster(
-        xmn = ext[1], xmx = ext[2], ymn = ext[3],
-        ymx = ext[4], matrix(x.dens$ud, nrow = R)
+      x.dens <- terra::rast(
+        matrix(x.dens$ud,nrow = 100)
+      )
+      terra::ext(x.dens)<-c(
+        xmin = ext[1], 
+        xmax = ext[2], 
+        ymin = ext[3],
+        ymax = ext[4]
       )
       if (!is.null(th)) {
-        th.value <- quantile(raster::extract(x.dens, x), th)
+        th.value <- quantile(terra::extract(x.dens, x)[,1], th)
         x.dens[x.dens < th.value] <- 0
       }
       if (!is.null(env.mask)) {
@@ -123,13 +128,13 @@ ecospat.kd <- function(x, ext, R = 100, th = 0, env.mask = c(),
                         xmin = ext[c(1, 3)],
                         xmax = ext[c(2, 4)], gridsize = c(R, R)
       )
-      x.dens <- raster::flip(raster::t(raster::raster(x.dens$estimate)), direction = "y")
-      raster::extent(x.dens) <- c(
-        xmn = ext[1], xmx = ext[2], ymn = ext[3],
-        ymx = ext[4]
+      x.dens <- terra::flip(terra::t(terra::rast(x.dens$estimate)), direction = "vertical")
+      terra::ext(x.dens) <- c(
+        xmin = ext[1], xmax = ext[2], ymin = ext[3],
+        ymax = ext[4]
       )
       if (!is.null(th)) {
-        th.value <- quantile(raster::extract(x.dens, x), th)
+        th.value <- quantile(terra::extract(x.dens, x)[,1], th)
         x.dens[x.dens < th.value] <- 0
       }
       if (!is.null(env.mask)) {
@@ -204,8 +209,8 @@ ecospat.grid.clim.dyn <- function(glob, glob1, sp, R = 100, th.sp = 0,
     
     glob1.dens <- ecospat.kd(x = glob1, ext = ext, method = kernel.method, th = th.env, R = R)
     if (!is.null(geomask)) {
-      raster::crs(geomask) <- NA
-      glob1.dens <- raster::mask(glob1.dens, geomask, updatevalue = 0) # Geographical mask in the case if the analysis takes place in the geographical space
+      terra::crs(geomask) <- NA
+      glob1.dens <- terra::mask(glob1.dens, geomask, updatevalue = 0) # Geographical mask in the case if the analysis takes place in the geographical space
     }
     sp.dens <- ecospat.kd(
       x = sp, ext = ext, method = kernel.method, th = th.sp, R =R,
@@ -215,12 +220,12 @@ ecospat.grid.clim.dyn <- function(glob, glob1, sp, R = 100, th.sp = 0,
     x <- seq(from = ext[1], to = ext[2], length.out = R)
     y <- seq(from = ext[3], to = ext[4], length.out = R)
     l$y <- y
-    Z <- glob1.dens * nrow(glob1) / raster::cellStats(glob1.dens, "sum") # rescale density to the number of occurrences in sp, ie. number of occurrence/pixel
-    z <- sp.dens * nrow(sp) / raster::cellStats(sp.dens, "sum") # rescale density to the number of occurrences in sp, ie. number of occurrence/pixel
-    z.uncor <- z / raster::cellStats(z, "max")
+    Z <- glob1.dens * nrow(glob1) / terra::global(glob1.dens, "sum")$sum # rescale density to the number of occurrences in sp, ie. number of occurrence/pixel
+    z <- sp.dens * nrow(sp) / terra::global(sp.dens, "sum")$sum  # rescale density to the number of occurrences in sp, ie. number of occurrence/pixel
+    z.uncor <- z / terra::global(z, "max")$max
     z.cor <- z / Z # correct for environment prevalence
     z.cor[is.na(z.cor)] <- 0 # remove n/0 situations
-    z.cor <- z.cor / raster::cellStats(z.cor, "max")
+    z.cor <- z.cor / terra::global(z.cor, "max")$max
   }
   
   w <- z.uncor # niche envelope
@@ -326,30 +331,29 @@ ecospat.plot.niche.dyn <- function(z1, z2, quant = 0, title = "", name.axis1 = "
   }
   
   if (!is.null(z1$y)) {
+    # assign the correct color to each category of the niche
+    col_category<-c("#FFFFFF",col.exp,col.unf,col.stab)[1+(sort(terra::unique(2*z1$w+z2$w)[,1]))]
     
     if (interest == 1) {
-      plot(z1$z.uncor,col=gray(100:0 / 100),legend=F, xlab = name.axis1, 
-           ylab = name.axis2,...)
+      terra::plot(z1$z.uncor,col=gray(100:0 / 100),legend=FALSE, xlab = name.axis1, 
+           ylab = name.axis2,mar = c(3.1,3.1,2.1,3.1))
     }
     if (interest == 2) {
-      plot(z2$z.uncor,col=gray(100:0 / 100),legend=F,xlab = name.axis1, 
-           ylab = name.axis2,...)
+      terra::plot(z2$z.uncor,col=gray(100:0 / 100),legend=FALSE,xlab = name.axis1, 
+           ylab = name.axis2,mar = c(3.1,3.1,2.1,3.1))
     }
-    
-    raster::image(2*z1$w+z2$w,col=c("#FFFFFF",col.exp,col.unf,col.stab), 
-                  add = TRUE,legend=FALSE)
+    terra::plot(2*z1$w+z2$w,col=col_category, 
+                 add = TRUE,legend=FALSE)
     
     title(title)
-    raster::contour(
+    terra::contour(
       z1$Z, add = TRUE, levels = quantile(z1$Z[z1$Z > 0], c(0, quant)),
       drawlabels = FALSE, lty = c(1, 2), col = colZ1
     )
-    raster::contour(
+    terra::contour(
       z2$Z, add = TRUE, levels = quantile(z2$Z[z2$Z > 0], c(0, quant)),
       drawlabels = FALSE, lty = c(1, 2), col = colZ2
     )
-    axis(1,labels = FALSE, lwd.ticks = 0);axis(2, lwd.ticks = 0,labels = FALSE)
-    axis(3,labels = FALSE, lwd.ticks = 0);axis(4, lwd.ticks = 0,labels = FALSE)
   }
 }
 
@@ -413,7 +417,7 @@ ecospat.niche.dyn.index <- function(z1, z2, intersection = NA) {
   
   dyn <- (-1 * z.exp.cat) + (2 * z.stable.cat) + z.res.cat
   if (ncol(w1) == 2) {
-    dyn <- raster(dyn)
+    dyn <- terra::rast(dyn)
   } # draw matrix with 3 categories of niche dynamic
   expansion.index.w <- sum(obs.exp) / sum(obs.stab + obs.exp) # expansion
   stability.index.w <- sum(obs.stab) / sum(obs.stab + obs.exp) # stability
@@ -442,7 +446,7 @@ ecospat.margin <- function(z, th.quant = 0, kern.method = "adehabitat",
     th = th.quant, env.mask = z$Z > 0, method = kern.method
   )
   niche.envelope <- (niche.dens > 0) / (niche.dens > 0)
-  niche.contour <- as(raster::rasterToPolygons(niche.envelope, dissolve = TRUE), "SpatialLines")
+  niche.contour <- terra::as.polygons(niche.envelope, dissolve = TRUE)
   
   if (bootstrap == T) {
     my.df <- dplyr::as_tibble(z$sp) # convert to tibble to do faster resampling
@@ -489,13 +493,10 @@ ecospat.margin <- function(z, th.quant = 0, kern.method = "adehabitat",
     sum.bst <- Reduce("+", bin.bst) ## sum of the binarized kernel distribution
     
     niche.envelope <- sum.bst / bootstrap.rep * 100 ## scaling in percentage
-    niche.contour <- as(raster::rasterToPolygons(niche.envelope >= 95, fun = function(x) {
-      x == 1
-    }, dissolve = TRUE), "SpatialLines")
+    niche.contour <- terra::as.polygons(niche.envelope >= 95, dissolve = TRUE)
   }
   return(list(
     niche.density = niche.dens, niche.envelope = niche.envelope,
     niche.contour = niche.contour
   ))
 }
-
